import Bot from '../lib/bot';
import Queue from './queue';
export declare type ErrorFunc<T> = (reply: string, parsed: T) => void;
export declare type ParseFunc<T> = (message: string) => T;
export declare type CheckFunc<T> = (parsed: T) => boolean;
export declare type ParserObject<T> = {
    parse: ParseFunc<T>;
    check: CheckFunc<T>;
};
export declare type Parser<T> = ParseFunc<T> | ParserObject<T>;
declare class Dialog<B extends Bot<any, {
    id: string;
}>> {
    bot: B;
    chat: string;
    static isDefault: boolean;
    static match(message: string): boolean;
    user: B['IUser'];
    queue: Queue;
    constructor(bot: B, chat: string);
    /**
     * All communication logic(send a message, parse, ask something) during dialog
     * lifecycle are located here. This method is called by `Bot` if user's message
     * is a start for this dialog. When `Promise` returned by `talk` is resolved,
     * `Bot` counts that as the end of the dialog and will not send next messages
     * to this dialog, but run `match` on dialogs and try to find next suitable
     * dialog.
     */
    talk(): Promise<void>;
    onIncomingMessage(message: string): void | Promise<void>;
    onOutgoingMessage(message: B['IMessage']): void | Promise<void>;
    onStart(): void | Promise<void>;
    onAbort(): void | Promise<void>;
    onEnd(): void | Promise<void>;
    /**
     * Format and send message to user.
     * To learn more about formatting, check [[formatting spec]]
     */
    message(message: B['IMessage']): Promise<any>;
    /**
     * Queue parse for user messages
     */
    parse<T>(parserFunc: ParseFunc<T>): any;
    parse<T>(parserFunc: ParseFunc<T>, errorMessage: B['IMessage']): Promise<T>;
    parse<T>(parserFunc: ParseFunc<T>, errorCallback: ErrorFunc<T>): Promise<T>;
    parse<T>(parserObject: ParserObject<T>, errorMessage: B['IMessage']): Promise<T>;
    parse<T>(parserObject: ParserObject<T>, errorCallback: ErrorFunc<T>): Promise<T>;
    ask<T>(message: B['IMessage'], parserFunc: ParseFunc<T>): Promise<T>;
    ask<T>(message: B['IMessage'], parserFunc: ParseFunc<T>, errorMessage: B['IMessage']): Promise<T>;
    ask<T>(message: B['IMessage'], parserFunc: ParseFunc<T>, errorCallback: ErrorFunc<T>): Promise<T>;
    ask<T>(message: B['IMessage'], parserObject: ParserObject<T>): Promise<T>;
    ask<T>(message: B['IMessage'], parserObject: ParserObject<T>, errorMessage: B['IMessage']): Promise<T>;
    ask<T>(message: B['IMessage'], parserObject: ParserObject<T>, errorCallback: ErrorFunc<T>): Promise<T>;
    startDialog(DialogClass: typeof Dialog, options?: {
        [key: string]: any;
    }): void;
}
export default Dialog;
